package jp.deadend.noname.mt

import android.content.Context
import android.graphics.Canvas
import androidx.appcompat.widget.AppCompatEditText
import android.util.AttributeSet
import android.view.MotionEvent

open class AppCompatEditTextWithFastScroller : AppCompatEditText {
    constructor(context: Context) : super(context)
    constructor(context: Context, attrs: AttributeSet) : super(context, attrs)
    constructor(context: Context, attrs: AttributeSet, defStyleAttr: Int) : super(context, attrs, defStyleAttr)

    private val mFastScroller: FastScroller by lazy { FastScroller(context, this) }

    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)

        mFastScroller.draw(canvas)
    }

//    override fun onSizeChanged(w: Int, h: Int, oldw: Int, oldh: Int) {
//        mFastScroller.onSizeChanged(w, h, oldw, oldh)
//    }

    override fun onScrollChanged(horiz: Int, vert: Int, oldHoriz: Int, oldVert: Int) {
        super.onScrollChanged(horiz, vert, oldHoriz, oldVert)

        mFastScroller.onScroll(
                vert, bottom - top - extendedPaddingBottom - extendedPaddingTop, layout.height
        )
    }

    override fun isVerticalScrollBarEnabled(): Boolean = mFastScroller.isVisible()

    override fun onTouchEvent(event: MotionEvent): Boolean {
        if (mFastScroller.onInterceptTouchEvent(event)) { return true }
        if (mFastScroller.onTouchEvent(event)) { return true }

        return super.onTouchEvent(event)
    }
}