package jp.deadend.noname.mt

import android.app.Application
import android.content.ContentResolver
import android.net.Uri
import android.provider.OpenableColumns
import androidx.preference.PreferenceManager
import java.io.File

val prefs: Prefs by lazy { MainApplication.prefs!! }

class MainApplication: Application() {
    companion object {
        var prefs: Prefs? = null
        val DEFAULT_SHORTCUT_KEYS = mapOf(
            EditorCommand.UNDO to 868, // Ctrl+z
            EditorCommand.REDO to 852, // Ctrl+y
            EditorCommand.GOTO_TOP to 122, // Home
            EditorCommand.GOTO_BOTTOM to 123, // End
            EditorCommand.GOTO_LINE_BEGINNING to 468, // Ctrl+a
            EditorCommand.GOTO_LINE_END to 532, // Ctrl+e
            EditorCommand.CUT_LINE to 628, // Ctrl+k
            EditorCommand.CUT to 836, // Ctrl+x;
            EditorCommand.COPY to 500, // Ctrl+c
            EditorCommand.PASTE to 804, // Ctrl+v
            EditorCommand.OPEN to 692, // Ctrl+o
            EditorCommand.SAVE to 756, // Ctrl+s
            EditorCommand.SAVE_AS to 757, // Ctrl+Shift+s
            EditorCommand.CLOSE to 820, // Ctrl+w
            EditorCommand.FIND to 548, // Ctrl+f
            EditorCommand.FIND_NEXT to 564, // Ctrl+g
            EditorCommand.FIND_PREV to 565 // Shift+Ctrl+g
        )
    }

    override fun onCreate() {
        prefs = Prefs(applicationContext)
        if (PreferenceManager
                .getDefaultSharedPreferences(applicationContext)
                .getString(getString(R.string.prefkey_shortcuts), null) == null) {
            prefs!!.storeShortcuts(DEFAULT_SHORTCUT_KEYS)
        }
        super.onCreate()
    }


    fun getFileName(uri: Uri): String? = when(uri.scheme) {
        ContentResolver.SCHEME_CONTENT -> getContentFileName(uri)
        else -> uri.path?.let { File(it) }?.name
    }

    private fun getContentFileName(uri: Uri): String? = runCatching {
        contentResolver.query(uri, null, null, null, null)?.use { cursor ->
            cursor.moveToFirst()
            return@use cursor.getColumnIndexOrThrow(OpenableColumns.DISPLAY_NAME).let { cursor.getString(it) }
        }
    }.getOrNull()
}