package jp.deadend.noname.mt

import android.content.Context
import androidx.preference.PreferenceManager
import java.lang.IllegalArgumentException
import java.lang.NumberFormatException
import java.lang.StringBuilder

class Prefs(context: Context) {
    private val prefs = PreferenceManager.getDefaultSharedPreferences(context)
    private val res = context.resources

    var fontSize: Int
        get() = prefs.getInt(res.getString(R.string.prefkey_font_size), res.getInteger(R.integer.pref_value_font_size_default))
        set(value) = prefs.edit().putInt(res.getString(R.string.prefkey_font_size), value).apply()

    var lineSpacing: Int
        get() = prefs.getInt(res.getString(R.string.prefkey_line_spacing), res.getInteger(R.integer.pref_value_line_spacing_default))
        set(value) = prefs.edit().putInt(res.getString(R.string.prefkey_line_spacing), value).apply()

    var useCustomFont: Boolean
        get() = prefs.getBoolean(res.getString(R.string.prefkey_use_custom_font), false)
        set(value) = prefs.edit().putBoolean(res.getString(R.string.prefkey_use_custom_font), value).apply()

    var customFontName: String
        get() = prefs.getString(res.getString(R.string.prefkey_custom_font_name), null) ?: ""
        set(value) = prefs.edit().putString(res.getString(R.string.prefkey_custom_font_name), value).apply()

    var showLineNumbers: Boolean
        get() = prefs.getBoolean(res.getString(R.string.prefkey_show_line_numbers), true)
        set(value) = prefs.edit().putBoolean(res.getString(R.string.prefkey_show_line_numbers), value).apply()

    var fullScreen: Boolean
        get() = prefs.getBoolean(res.getString(R.string.prefkey_fullscreen), false)
        set(value) = prefs.edit().putBoolean(res.getString(R.string.prefkey_fullscreen), value).apply()

    var lastUri: String?
        get() = prefs.getString(res.getString(R.string.prefkey_last_uri), null)
        set(value) = prefs.edit().putString(res.getString(R.string.prefkey_last_uri), value).apply()

    var lastOffset: Int
        get() = prefs.getInt(res.getString(R.string.prefkey_last_offset), 0)
        set(value) = prefs.edit().putInt(res.getString(R.string.prefkey_last_offset), value).apply()

    var colorBackground: Int
        get() = prefs.getInt(res.getString(R.string.prefkey_color_background), 0xFFFFFFFF.toInt())
        set(value) = prefs.edit().putInt(res.getString(R.string.prefkey_color_background), value).apply()

    var colorCurrentLine: Int
        get() = prefs.getInt(res.getString(R.string.prefkey_color_currentline), 0xFFCCCCCC.toInt())
        set(value) = prefs.edit().putInt(res.getString(R.string.prefkey_color_currentline), value).apply()

    var colorText: Int
        get() = prefs.getInt(res.getString(R.string.prefkey_color_text), 0xFF000000.toInt())
        set(value) = prefs.edit().putInt(res.getString(R.string.prefkey_color_text), value).apply()

    private var shortcutsStr: String
        get() = prefs.getString(res.getString(R.string.prefkey_shortcuts), null) ?: ""
        set(value) = prefs.edit().putString(res.getString(R.string.prefkey_shortcuts), value).apply()

    fun storeShortcuts(map: Map<EditorCommand, Int>) {
        val prefStr = StringBuilder()
        for ((key, value) in map) {
            prefStr.let {
                it.append(value.toString())
                it.append(",")
                it.append(key.name)
                it.append(":")
            }
            shortcutsStr = prefStr.toString()
        }
    }

    fun getShortcuts(): MutableMap<Int, EditorCommand> {
        val result = mutableMapOf<Int, EditorCommand>()
        val prefStr = jp.deadend.noname.mt.prefs.shortcutsStr
        if (prefStr.isNotEmpty()) {
            val shortcuts = prefStr.split(":").filterNot { it.isEmpty() }
            for (shortcut in shortcuts) {
                try {
                    val tmp = shortcut.split(",").filterNot { it.isEmpty() }
                    result[tmp[0].toInt()] = EditorCommand.valueOf(tmp[1])
                } catch (e: IllegalArgumentException) {
                } catch (e: NumberFormatException) {
                }
            }
        }

        return result
    }

//    var encoding: String
//        get() = prefs.getString(res.getString(R.string.pref_key_encoding), null) ?: res.getString(R.string.pref_value_encoding_default)
//        set(value) = prefs.edit().putString(res.getString(R.string.pref_key_encoding), value).apply()
//
//    var newLine: String
//        get() {
//            val key = res.getString(R.string.pref_key_newline)
//            val defaultVal = res.getString(R.string.pref_value_newline_default)
//            val str = prefs.getString(key, null) ?: defaultVal
//            return try {
//                NewLine.valueOf(str).value
//            } catch (e: IllegalArgumentException) {
//                prefs.edit().remove(key).apply()
//                defaultVal
//            }
//        }
//        set(value) = prefs.edit().putString(res.getString(R.string.pref_key_newline), value).apply()
}

