package jp.deadend.noname.mt

import android.content.Context
import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.TextView
import jp.deadend.noname.mt.databinding.ActivitySetKeyBinding

class SetKeyActivity : AppCompatActivity() {
    private var mShortcutKeys = mutableMapOf<EditorCommand, Int>()
    private var mLastClickedItem: EditorCommand? = null
    private lateinit var binding: ActivitySetKeyBinding

    private inner class SetKeyAdapter(
            context: Context,
            objects: Array<EditorCommand>
    ) : ArrayAdapter<EditorCommand>(context, 0, objects) {
        private val mLayoutInflater: LayoutInflater = LayoutInflater.from(context)

        override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
            val view = convertView ?: mLayoutInflater
                    .inflate(R.layout.setkey_row, parent, false)
                    .apply {
                        tag = ViewHolder(
                            this.findViewById(R.id.textViewMain),
                            this.findViewById(R.id.textViewSub)
                        )
                    }
            val holder = view.tag as ViewHolder

            val item = getItem(position)
            holder.main.text = if (item != null) getString(item.value) else ""
            item?.let {
                if (mShortcutKeys.containsKey(it)) {
                    val key = requireNotNull(mShortcutKeys[it]) { "SetKeyAdapter BUG: item is null!" }
                    holder.sub.text = SetKeyDialogFragment.getKeyName(key)
                } else {
                    holder.sub.text = ""
                }
            }

            return view
        }

        private inner class ViewHolder (val main: TextView, val sub: TextView)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivitySetKeyBinding.inflate(layoutInflater)
        setContentView(binding.root)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        val commands = EditorCommand.values()

        binding.listView.adapter = SetKeyAdapter(this, commands)
        binding.listView.onItemClickListener = AdapterView.OnItemClickListener { parent, _, position, _ ->
            val item = parent.getItemAtPosition(position) as EditorCommand
            val key = if (mShortcutKeys.containsKey(item)) {
                requireNotNull(mShortcutKeys[item]) { "This should not be happen." }
            } else {
                -1
            }

            mLastClickedItem = item
            val ft = supportFragmentManager.beginTransaction()
            val fragment = SetKeyDialogFragment.newInstance(key)
            fragment.show(ft, "dialog")
        }

        binding.buttonReset.setOnClickListener {
            val dialog = ConfirmationDialogFragment.newInstance(getString(R.string.message_confirm_reset_to_default))
            dialog.setListener(
                    object : ConfirmationDialogFragment.Listener {
                        override fun onPositiveClick() {
                            mShortcutKeys.clear()
                            for ((key, value) in MainApplication.DEFAULT_SHORTCUT_KEYS) {
                                mShortcutKeys[key] = value
                            }
                            binding.listView.invalidateViews()
                        }

                        override fun onNegativeClick() {}
                    })
            dialog.show(supportFragmentManager, "dialog")
        }
    }

    override fun onResume() {
        super.onResume()

        mShortcutKeys.clear()
        val map = prefs.getShortcuts()
        for ((key, cmd) in map) { mShortcutKeys[cmd] = key }
    }

    override fun onPause() {
        super.onPause()

        prefs.storeShortcuts(mShortcutKeys)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            android.R.id.home -> finish()
            else -> return super.onOptionsItemSelected(item)
        }

        return true
    }

    fun setKey(key: Int) {
        mLastClickedItem?.let {
            if (mShortcutKeys.containsValue(key)) {
                for (cmd in mShortcutKeys.keys) {
                    if (mShortcutKeys[cmd] == key && cmd !== it) {
                        val dialog = SimpleMessageDialogFragment.newInstance(
                                getString(R.string.error_shortcutkey_exists)
                        )
                        dialog.show(supportFragmentManager, "dialog")
                        return
                    }
                }
            }

            mShortcutKeys[it] = key
            binding.listView.invalidateViews()
        }
    }

    fun clearKey() {
        mLastClickedItem?.let {
            mShortcutKeys.remove(it)
            binding.listView.invalidateViews()
        }
    }
}
