package jp.deadend.noname.mt

import android.content.Context
import android.net.Uri
import android.os.Bundle
import android.view.Menu
import android.view.MenuItem
import androidx.activity.result.contract.ActivityResultContracts
import androidx.appcompat.app.AppCompatActivity
import androidx.preference.Preference
import androidx.preference.PreferenceFragmentCompat
import androidx.preference.PreferenceManager

class SettingsActivity : AppCompatActivity() {
    class SettingsFragment : PreferenceFragmentCompat() {
        private val chooseFontFileActivity =
            registerForActivityResult(ActivityResultContracts.OpenDocument()) { uri ->
                if (uri != null) {
                    val filename = (activity?.application as MainApplication?)?.getFileName(uri)
                    filename?.let {
                        loadFontFromUri(uri, it)
                        prefs.customFontName = it
                        findPreference<Preference>(getString(R.string.prefkey_use_custom_font))?.summary = it
                    }
                }
            }

        private fun loadFontFromUri(uri: Uri, fileName: String) {
            try {
                val input = requireContext().contentResolver?.openInputStream(uri)?.buffered()
                val output = requireContext().openFileOutput(fileName, Context.MODE_PRIVATE).buffered()
                input?.use { i ->
                    output.use { o-> i.copyTo(o) }
                }
            } catch (e: Exception) {
                Trace.d("loadFontFromUri(): $e")
            }
        }

        override fun onCreatePreferences(savedInstanceState: Bundle?, rootKey: String?) {
            addPreferencesFromResource(R.xml.prefs)
            findPreference<Preference>(getString(R.string.prefkey_use_custom_font))
                ?.summary = prefs.customFontName
        }

        override fun onPreferenceTreeClick(preference: Preference): Boolean {
            if (preference.key == getString(R.string.prefkey_custom_font_name)) {
                chooseFontFileActivity.launch(arrayOf("*/*"))
                return true
            }

            return super.onPreferenceTreeClick(preference)
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        title = getString(R.string.label_pref_activity)

        supportFragmentManager.beginTransaction()
            .replace(android.R.id.content, SettingsFragment())
            .commit()
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        menuInflater.inflate(R.menu.menu_settings, menu)

        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            R.id.menu_settings_clear_prefs -> {
                val dialog = ConfirmationDialogFragment.newInstance(getString(R.string.message_confirm_clear_prefs))
                dialog.setListener(
                    object : ConfirmationDialogFragment.Listener {
                        override fun onPositiveClick() {
                            PreferenceManager.getDefaultSharedPreferences(applicationContext)
                                .edit()
                                .clear()
                                .apply()
                            supportFragmentManager.beginTransaction()
                                .replace(android.R.id.content, SettingsFragment())
                                .commit()
                        }

                        override fun onNegativeClick() {}
                    })
                dialog.show(supportFragmentManager, "dialog")
            }
            android.R.id.home -> finish()
        }

        return super.onOptionsItemSelected(item)
    }

}